#Plotting spatial patterns
library(ncdf4)
library(ggplot2)
library(gridExtra)
library(fields)
library(mapdata)
library(matrixStats)
library(metR)
library(RColorBrewer)

rm(list=ls()) ; gc()

#Directory information
setwd("../../../scratch/")
OUTPUTS <- "/esarchive/scratch/molmo/CPs_SLP_ERA5/"

#Useful functions
png.save <- function(x,filename){
  png(filename = paste(filename,".png",sep=""))
  plot(x)
  dev.off()
}

#########################################################
#Load ERA5 data
message("Loading and preprocessing data...")
nc <- nc_open("/esarchive/scratch/molmo/g500_5022_50km_anom.nc")
nc_lon <- ncvar_get(nc,"lon")
nc_lat <- ncvar_get(nc,"lat")
nc_var <- ncvar_get(nc,"g500") #x,y,time

gc()

#Selection of spatial domains
gridsize_x <- abs(nc_lon[2]-nc_lon[1])
gridsize_y <- abs(nc_lat[2]-nc_lat[1])

domain1 <- list(lon <- which(nc_lon %in% seq(-15,15,by=gridsize_x)),
                lat <- which(nc_lat %in% seq(32,48,by=gridsize_y))) #red domain D1 #Spain

domain2 <- list(lon <- which(nc_lon %in% seq(-10,25,by=gridsize_x)),
                lat <- which(nc_lat %in% seq(33,45,by=gridsize_y))) #red domain D2 PEP Mediterranean

domain3 <- list(lon <- which(nc_lon %in% seq(-35,35,by=gridsize_x)),
                lat <- which(nc_lat %in% seq(20,70,by=gridsize_y))) #red domain D3 #Europe

domain <- domain3 #; dom = "D3"
nc_var <- nc_var[domain[[1]],domain[[2]],] ; gc()

#Organize and Standardize data
var <- matrix(nc_var,dim(nc_var)[1]*dim(nc_var)[2],dim(nc_var)[3]) #space,time
rm(nc_var) ; gc()

var[which(var< -20000)] <- NA
gc()
var[which(var> 20000)] <- NA

#Load clustering information
#k=3; dom = "D1" ; x1 = -15 ; x2 = 15 ; y1 = 32 ; y2 = 48
#k=5 ; dom = "D2" ; x1 = -10; x2 = 25; y1 = 33; y2 = 45
#k=7; dom = "D3" ; x1 = -35 ; x2 = 35 ; y1 = 20 ; y2 = 70
do <- c("D1","D2","D3")

for(d in 1:3){
  dom <- do[d]
  
  for(k in c(3,4,5,7)){
    
    
    load(file = paste("WardCluster_Dendrogram_",dom,"_50km.Rdata",sep=""))
    vector_cluster <- cutree(ward_clustering,k)
    
    #Preparing data
    xmn=-35;xmx=35 ; ymn=20;ymx=70
    longitude=rep(seq(xmn,xmx,gridsize_x),length(domain[[2]]))
    latitude=rep(seq(ymn,ymx,gridsize_x),each=length(domain[[1]]))
    
    Z <- list() 
    
    for(cp in 1:k){
      Z[[cp]] <- rowMeans(var[,which(vector_cluster==cp)],na.rm=T)
    }
    
    datitos <- data.frame(Longitude=rep(longitude,k),
                          Latitude=rep(latitude,k),
                          Z = unlist(Z)/10,
                          CP = rep(1:k,each=length(unlist(Z))/k))
    datitos$CP <- factor(datitos$CP,levels=c(1:k))
    
    datitos$Z[which(datitos$Z>150)] <- 149.9
    datitos$Z[which(datitos$Z< -150)] <- -149.9
    
    p <- ggplot(datitos,aes(x=Longitude,y=Latitude))  + geom_contour_fill(aes(z=Z),size=1,breaks = seq(-150,150,by=15),kriging = F)  + borders(colour = "grey22",size=0.3) + facet_wrap(~CP,nrow=1)+ 
      scale_fill_gradientn(name=expression("m"),
                           colours=rev(brewer.pal(11,"RdBu")),aesthetics=c("fill"),
                           na.value = "white",limits=c(-150,150)) +
      coord_quickmap(xlim=c(min(datitos$Longitude),max(datitos$Longitude)),ylim=c(min(datitos$Latitude),max(datitos$Latitude)),expand = F) +
      xlab("Longitude") + ylab("Latitude") + theme_bw() #+#+ ggtitle("Patterns") + 
     # geom_contour2(aes(z=Z),  color="black", na.rm = T, binwidth=5)+
      #geom_text_contour(aes(z=Z),stroke = 0.10, rotate = T, check_overlap = T, binwidth=50,size=2.5)
    #p <- p + geom_rect(aes(xmin=x1,xmax=x2,ymin=y1,ymax=y2),col="red",fill=NA)
    #p <- p + geom_rect(aes(xmin=0,xmax=3,ymin=40,ymax=43),col="black",fill="grey",alpha=0.01)
    ggsave(p,filename=paste(OUTPUTS,"SpatialPatterns_Ward_Z500_Anom",dom,"_",k,"CPs.png",sep=""),device="png",
           dpi=600,width=2*k,height = 3)
    
    ##################################################################
  }
}
